//
//  UpnpManager.m
//  uPnP
//
//  Created by rk on 5/12/10.
//  Copyright 2010 Interchart. All rights reserved.
//

//#import <SystemConfiguration/SystemConfiguration.h>

#import "UpnpManager.h"
//#import "UpnpDevice.h"

@implementation UpnpManager


- (NSMutableArray *) discoverDevices {
	struct UPNPDev * devList = 0;
	const char * multicastif = 0;
	const char * minissdpdpath = 0;
	
	devList = upnpDiscover(3000, multicastif, minissdpdpath, 0);
	
	NSMutableArray *result = [NSMutableArray array];
	[result retain];
	if (devList != nil) {
		UpnpDev *dev = devList;
		while (dev != nil) {
			NSURL *deviceUrl = [[NSURL URLWithString:[NSString stringWithUTF8String:dev->descURL]] retain];
			UpnpDeviceType deviceType = 0;//UpnpDeviceType.Unknown;
			UpnpDevice *device = [[UpnpDevice alloc] init];
			[device retain];
			device._url = deviceUrl;
			device._deviceType = deviceType;
			//				switch (device->st[0]) {
			//					case 'a':
			//						
			//						break;
			//				}
			[result addObject:device];
			free(dev);
			dev = dev->pNext;
		}
		devList = nil;
	}	
	// result contains some UpnpDevice's. Fetch their URLs and find out what 
	// type of UpnpDevice they are (asynchronously via event loop)
	//[self fetchDeviceInfos: result];
	return result;
}



#pragma mark -

+ (NSString *) listDevices: (struct UPNPDev *) deviceList {
	NSString *result = @"";
	if(deviceList) {
		struct UPNPDev * device;
		//struct UPNPUrls urls;
		//struct IGDdatas data;		
		result = [NSString stringWithFormat:@"%@", @"List of UPNP devices found on the network :\n"];
		for(device = deviceList; device; device = device->pNext) {
			result = [result stringByAppendingFormat:@" desc: %s\n st: %s\n\n",
							device->descURL, device->st];
		}
//		// TCMUPNPortMapper.m:170
		NSMutableArray *URLsToTry = [NSMutableArray array];
		NSMutableSet *triedURLSet = [NSMutableSet set];
		
		bool foundIDGDevice = FALSE;
		for(device = deviceList; device; device = device->pNext) {
			NSURL *descURL = [NSURL URLWithString:[NSString stringWithUTF8String:device->descURL]];
            //SCNetworkConnectionFlags status;
			//Boolean success = SCNetworkCheckReachabilityByName([[descURL host] UTF8String], &status); 
			SCNetworkReachabilityRef   target;
			SCNetworkReachabilityFlags flags = 0;			
			target = SCNetworkReachabilityCreateWithName(NULL, [[descURL host] UTF8String]);
			Boolean success = SCNetworkReachabilityGetFlags(target, &flags);
			CFRelease(target);
			
			
#ifndef NDEBUG
			//NSLog(@"UPnP: %@ %c%c%c%c%c%c%c host:%s st:%s",
			//					  success ? @"YES" : @" NO",
			//					  (status & kSCNetworkFlagsTransientConnection)  ? 't' : '-',
			//					  (status & kSCNetworkFlagsReachable)            ? 'r' : '-',
			//					  (status & kSCNetworkFlagsConnectionRequired)   ? 'c' : '-',
			//					  (status & kSCNetworkFlagsConnectionAutomatic)  ? 'C' : '-',
			//					  (status & kSCNetworkFlagsInterventionRequired) ? 'i' : '-',
			//					  (status & kSCNetworkFlagsIsLocalAddress)       ? 'l' : '-',
			//					  (status & kSCNetworkFlagsIsDirect)             ? 'd' : '-',
			//					  device->descURL,
			//					  device->st
			//					  );
			NSLog(@"UPnP: %@ %c%c%c%c%c%c%c host:%s st:%s",
								  success ? @"YES" : @" NO",
								  (flags & kSCNetworkFlagsTransientConnection)  ? 't' : '-',
								  (flags & kSCNetworkFlagsReachable)            ? 'r' : '-',
								  (flags & kSCNetworkFlagsConnectionRequired)   ? 'c' : '-',
								  (flags & kSCNetworkFlagsConnectionAutomatic)  ? 'C' : '-',
								  (flags & kSCNetworkFlagsInterventionRequired) ? 'i' : '-',
								  (flags & kSCNetworkFlagsIsLocalAddress)       ? 'l' : '-',
								  (flags & kSCNetworkFlagsIsDirect)             ? 'd' : '-',
								  device->descURL,
								  device->st
								  );
#endif
				// only connect to directly reachable hosts which we haven't tried yet (if you are multihoming then you get all of the announcement twice
                //if (success && (status & kSCNetworkFlagsIsDirect)) {
			if (success && (flags & kSCNetworkFlagsIsDirect)) {
                if (![triedURLSet containsObject:descURL]) {
                    [triedURLSet addObject:descURL];
                    //if ([[descURL host] isEqualToString:[[TCMPortMapper sharedInstance] routerIPAddress]]) {
//                        [URLsToTry insertObject:descURL atIndex:0];
//                    } else {
                        [URLsToTry addObject:descURL];
                    //}
                }
            }
		}
		NSEnumerator *URLEnumerator = [URLsToTry objectEnumerator];
        NSURL *descURL = nil;
		NSString *errorString;
		Boolean didFail;
		char externalIPAddress[16];
		
        while ((descURL = [URLEnumerator nextObject])) {
			char lanaddr[16];	/* our ip address on the LAN */
			struct IGDdatas _igddata;
			struct UPNPUrls _urls;
#ifndef NDEBUG
            NSLog(@"UPnP: trying URL:%@",descURL);
#endif
//			// freeing the url still seems like a good idea - why isn't it?
//			if (_urls.controlURL) FreeUPNPUrls(&_urls);
//			// get the new control URLs - this call mallocs the control URLs
            if (UPNP_GetIGDFromUrl([[descURL absoluteString] UTF8String],&_urls,&_igddata,lanaddr,sizeof(lanaddr))) {
                int r = UPNP_GetExternalIPAddress(_urls.controlURL,
									  _igddata.servicetype,
									  externalIPAddress);
                if(r != 0 /* UPNPCOMMAND_SUCCESS */) {
                    didFail = YES;
                    errorString = [NSString stringWithFormat:@"GetExternalIPAddress() returned %d", r];
                } else {
                    if(externalIPAddress[0]) {
                        NSString *ipString = [NSString stringWithUTF8String:externalIPAddress];
                        NSMutableDictionary *userInfo = [NSMutableDictionary dictionaryWithObject:ipString forKey:@"externalIPAddress"];
                        NSString *routerName = [NSString stringWithUTF8String:_igddata.modeldescription];
                        if (routerName) [userInfo setObject:routerName forKey:@"routerName"];
                        //[[NSNotificationCenter defaultCenter] postNotificationOnMainThread:[NSNotification notificationWithName:TCMUPNPPortMapperDidGetExternalIPAddressNotification object:self userInfo:userInfo]];
                        foundIDGDevice = YES;
                        didFail = NO;
                        break;
					} else {
                        didFail = YES;
                        errorString = @"No external IP address!";
                    }
				}
			}
//            }
//			
		}
	}
	else {
		result = [NSString stringWithFormat:@"%@", @"No uPnP devices discovered.\n"];
	}
	return result;
}


@end
